### R code from vignette source 'genphenManual.Rnw'
### Encoding: UTF-8

###################################################
### code chunk number 1: genphenManual.Rnw:342-352
###################################################
require(genphen)
require(ggplot2)
require(knitr)
require(ggrepel)
require(reshape)
require(ape)
require(xtable)
options(xtable.floating = FALSE)
# Set CX14
Sys.setenv(USE_CXX14 = 1)


###################################################
### code chunk number 2: genphenManual.Rnw:358-363
###################################################
# genotype as matrix (120x154), we will subset part of it:
data(genotype.saap)

# phenotype as vector (120 measurements):
data(phenotype.saap)


###################################################
### code chunk number 3: genphenManual.Rnw:374-383
###################################################
# Format the genotype-phenotype data, such that it can then
# be visualized with ggplot
df <- data.frame(genotype.saap[, 82:89], 
                 phenotype = phenotype.saap,
                 stringsAsFactors = FALSE)
df <- melt(data = df, id.vars = "phenotype")
colnames(df) <- c("phenotype", "site", "genotype")
df$site <- gsub(pattern = "X", replacement = '', x = df$site)
df$site <- factor(x = df$site, levels = unique(df$site))


###################################################
### code chunk number 4: genphenManual.Rnw:387-399
###################################################
# Visualization
g <- ggplot(data = df)+
  facet_wrap(facets = ~site, nrow = 2, scales = "free_x")+
  geom_violin(aes(x = genotype, y = phenotype))+
  ylab(label = "Continuous phenotype")+
  xlab(label = "Genotypes")+
  geom_point(aes(x = genotype, y = phenotype, col = genotype), 
             size = 1, shape = 21, position = position_jitterdodge())+
  scale_color_discrete(name = "genotype")+
  theme_bw(base_size = 14)+
  theme(legend.position = "none")
g


###################################################
### code chunk number 5: genphenManual.Rnw:405-406
###################################################
g


###################################################
### code chunk number 6: genphenManual.Rnw:436-450
###################################################
# Run genphen
c.out <- genphen::runGenphen(genotype = genotype.saap[, 82:89], 
                             phenotype = phenotype.saap,
                             phenotype.type = "continuous",
                             model.type = "hierarchical",
                             mcmc.chains = 2,
                             mcmc.iterations = 1500,
                             mcmc.warmup = 500,
                             cores = 2,
                             hdi.level = 0.95,
                             stat.learn.method = "rf",
                             cv.iterations = 200,
                             rpa.iterations = 5,
                             with.stan.obj = TRUE)


###################################################
### code chunk number 7: genphenManual.Rnw:465-487
###################################################
# Get the scores data
c.score <- c.out$scores

# Some optional formatting for the SNPs (label = site : genotype1 -> genotype2)
c.score$label <- paste(c.score$site, ":", c.score$g1,
                       "->", c.score$g0, sep = '')

# Visualization
g <- ggplot(data = c.score)+
  geom_errorbar(aes(x = ca, ymin = beta.L, ymax = beta.H),
                width = 0.015, col = "darkgray")+
  geom_point(aes(x = ca, y = beta.mean, fill = kappa), shape = 21, size = 4)+
  geom_text_repel(aes(x = ca, y = beta.mean, label = label), size = 5)+
  theme_bw(base_size = 14)+
  ylab(label = "Slope coefficient (with 95% HDI)")+
  scale_x_continuous(name = "CA", limits = c(0, 1.05))+
  geom_hline(yintercept = 0, linetype = "dashed")+
  theme(legend.position = "top")+
  scale_fill_distiller(palette = "Spectral", limits = c(-0.2, 1))+
  guides(fill = guide_colorbar(barwidth = 10, barheight = 1.5))

g


###################################################
### code chunk number 8: genphenManual.Rnw:495-496
###################################################
g


###################################################
### code chunk number 9: genphenManual.Rnw:504-524
###################################################
# Description:
# Rounds digits to 2-decimal points, and concatinates the lower and upper
# limits of the HDI to have a simpler visualization
getHdiPretty <- function(x, digits = 2) {
  x[1] <- round(x = x[1], digits = digits)
  x[2] <- round(x = x[2], digits = digits)
  return(paste("(", x[1], ", ", x[2], ")", sep = ''))
}
c.score$beta.hdi <- apply(X = c.score[, c("beta.L", "beta.H")],
                          MARGIN = 1, getHdiPretty, digits = 2)
c.score$ca.hdi <- apply(X = c.score[, c("ca.L", "ca.H")],
                          MARGIN = 1, getHdiPretty, digits = 2)
c.score$kappa.hdi <- apply(X = c.score[, c("kappa.L", "kappa.H")],
                          MARGIN = 1, getHdiPretty, digits = 2)

# Print table
print(xtable(c.score[, c("label", "beta.mean", "beta.hdi", "ca",
                         "ca.hdi", "kappa", "kappa.hdi", "bc"), ],
             align = rep(x = "c", times = 9, digits = 2)),
      include.rownames = FALSE, size = "scriptsize")


###################################################
### code chunk number 10: genphenManual.Rnw:529-533
###################################################
print(xtable(c.score[, c("label", "beta.mean", "beta.hdi", "ca",
                         "ca.hdi", "kappa", "kappa.hdi", "bc"), ],
             align = rep(x = "c", times = 9, digits = 2)),
      include.rownames = FALSE, size = "scriptsize")


###################################################
### code chunk number 11: genphenManual.Rnw:548-559
###################################################
# Get the convergence output
c.convergence <- c.out$convergence
c.convergence$label <- paste(c.convergence$site, ":",
                             c.convergence$g1, "->",
                             c.convergence$g0, sep = '')
c.convergence$label[c.convergence$site == ''] <- ''

# Print table
print(xtable(c.convergence[, c("label", "par", "Rhat", "n_eff")],
             align = rep(x = "c", times = 5, digits = 2)),
      include.rownames = FALSE, size = "scriptsize")


###################################################
### code chunk number 12: genphenManual.Rnw:564-567
###################################################
print(xtable(c.convergence[, c("label", "par", "Rhat", "n_eff")],
             align = rep(x = "c", times = 5, digits = 2)),
      include.rownames = FALSE, size = "scriptsize")


###################################################
### code chunk number 13: genphenManual.Rnw:576-581
###################################################
# We can visualize the different MCMC properties as follows:
s1 <- rstan::stan_diag(object = c.out$stan.obj, information = "sample")
s2 <- rstan::stan_diag(object = c.out$stan.obj, information = "stepsize")
s3 <- rstan::stan_diag(object = c.out$stan.obj, information = "treedepth")
s4 <- rstan::stan_diag(object = c.out$stan.obj, information = "divergence")


###################################################
### code chunk number 14: genphenManual.Rnw:593-617
###################################################
# Get the ppc output
c.ppc <- c.out$ppc

# Optional setting of the min/max axis in order to make 
# the plot symmetric
min.axis <- min(c.ppc$pred.1.L, c.ppc$pred.0.L)
max.axis <- max(c.ppc$pred.1.H, c.ppc$pred.0.H)

# Visualization
g <- ggplot(data = c.ppc)+
  geom_errorbar(aes(x = pred.mean.1, ymin = pred.1.L, ymax = pred.1.H),
                col = "darkgray")+
  geom_errorbar(aes(x = pred.mean.0, ymin = pred.0.L, ymax = pred.0.H),
                col = "darkgray")+
  geom_point(aes(x = pred.mean.1, y = real.mean.1))+
  geom_point(aes(x = pred.mean.0, y = real.mean.0))+
  geom_abline(slope = 1, intercept = 0, linetype = "dashed", col = "red")+
  xlim(c(min.axis, max.axis))+
  ylim(c(min.axis, max.axis))+
  ylab(label = "Predicted mean phenotype (with 95% HDI)")+
  xlab(label = "Real mean phenotype")+
  theme_bw(base_size = 14)

g


###################################################
### code chunk number 15: genphenManual.Rnw:624-625
###################################################
g


###################################################
### code chunk number 16: genphenManual.Rnw:636-645
###################################################
# Get the RPA output
c.rpa <- c.out$rpa
c.rpa$label <- paste(c.rpa$site, ":", c.rpa$g1, "->", c.rpa$g0, sep = '')

g <- ggplot(data = c.rpa)+
  geom_text_repel(aes(x = rpa.power.error, y = rpa.sign.error, label = label),
                  arrow = arrow(length = unit(0.02, "npc")), size = 4)+
  theme_bw(base_size = 14)
g


###################################################
### code chunk number 17: genphenManual.Rnw:652-653
###################################################
g


###################################################
### code chunk number 18: genphenManual.Rnw:659-663
###################################################
print(xtable(c.rpa[, c("label",  "rpa.power.error", "rpa.sign.error",
                       "rpa.beta.mean", "rpa.beta.sd", "rpa.N")],
             align = rep(x = "c", times = 7, digits = 2)),
      include.rownames = FALSE, size = "small")


###################################################
### code chunk number 19: genphenManual.Rnw:668-672
###################################################
print(xtable(c.rpa[, c("label",  "rpa.power.error", "rpa.sign.error",
                       "rpa.beta.mean", "rpa.beta.sd", "rpa.N")],
             align = rep(x = "c", times = 7, digits = 2)),
      include.rownames = FALSE, size = "scriptsize")


###################################################
### code chunk number 20: genphenManual.Rnw:682-702
###################################################
# Compute the phylogenetic bias
bias <- runPhyloBiasCheck(genotype = genotype.saap,
                          input.kinship.matrix = NULL)

# Extract kinship matrix
kinship.matrix <- bias$kinship.matrix

# Extract the bias associated with mutations of the sites which 
# were included in the association analysis
mutation.bias <- bias$bias

# To make site id concordant with data
mutation.bias$site <- mutation.bias$site - 81
mutation.bias <- merge(x = c.score, y = mutation.bias,
                       by = c("site", "g1", "g0"))

# Show the bias table
print(xtable(mutation.bias[, c("site", "g1", "g0", "bias.g1", "bias.g0")],
             align = rep(x = "c", times = 6, digits = 2)),
      include.rownames = FALSE, size = "small")


###################################################
### code chunk number 21: genphenManual.Rnw:707-710
###################################################
print(xtable(mutation.bias[, c("site", "g1", "g0", "bias.g1", "bias.g0")],
             align = rep(x = "c", times = 6, digits = 2)),
      include.rownames = FALSE, size = "scriptsize")


###################################################
### code chunk number 22: genphenManual.Rnw:721-738
###################################################
color.a <- character(length = nrow(genotype.saap))
color.a[1:length(color.a)] <- "gray"
color.a[which(genotype.saap[, 82] == "h")] <- "orange"
color.a[which(genotype.saap[, 82] == "q")] <- "blue"

color.b <- character(length = nrow(genotype.saap))
color.b[1:length(color.b)] <- "gray"
color.b[which(genotype.saap[, 84] == "a")] <- "orange"
color.b[which(genotype.saap[, 84] == "d")] <- "blue"

c.hclust <- hclust(as.dist(kinship.matrix), method = "average")

par(mfrow = c(1, 2), mar = c(0,0,1,0) + 0.1)
plot(as.phylo(c.hclust), tip.color = color.a, cex = 0.6, 
     type = "fan", main = "B = 0.15")
plot(as.phylo(c.hclust), tip.color = color.b, cex = 0.6, 
     type = "fan", main = "B = 0.43")


###################################################
### code chunk number 23: genphenManual.Rnw:746-751
###################################################
par(mfrow = c(1, 2), mar = c(0,0,1,0) + 0.1)
plot(as.phylo(c.hclust), tip.color = color.a, cex = 0.6, 
     type = "fan", main = "B = 0.15")
plot(as.phylo(c.hclust), tip.color = color.b, cex = 0.6, 
     type = "fan", main = "B = 0.43")


###################################################
### code chunk number 24: genphenManual.Rnw:766-774
###################################################
# Genotype inputs:
data(genotype.snp)
# Select a single SNP
genotype.snp <- genotype.snp[, 10]

# Generate phenotype inputs:
dichotomous.phenotype.snp <- c(rbinom(n = 20, size = 1, prob = 0.25),
                               rbinom(n = 31, size = 1, prob = 0.75))


###################################################
### code chunk number 25: genphenManual.Rnw:779-786
###################################################
# Format the genotype-phenotype data, such that it can then
# be visualized with ggplot
df <- data.frame(genotype.snp,
                 phenotype = dichotomous.phenotype.snp,
                 stringsAsFactors = FALSE)
df <- melt(data = df, id.vars = "phenotype")
colnames(df) <- c("phenotype", "site", "genotype")


###################################################
### code chunk number 26: genphenManual.Rnw:790-802
###################################################
# Visualization
g <- ggplot(data = df)+
  geom_point(aes(x = genotype, y = phenotype, col = genotype), size = 1,
             shape = 21, position = position_jitterdodge(jitter.width = 0.2,
                                                         jitter.height = 0.1,
                                                         dodge.width = 0.5))+
  xlab(label = "Genotypes")+
  scale_y_continuous(name = "Phenotype", breaks = c(0, 1), labels = c(0, 1))+
  theme_bw(base_size = 14)+
  theme(legend.position = "none")

g


###################################################
### code chunk number 27: genphenManual.Rnw:808-809
###################################################
g


###################################################
### code chunk number 28: genphenManual.Rnw:838-852
###################################################
# run genphen
d.out <- genphen::runGenphen(genotype = genotype.snp,
                             phenotype = dichotomous.phenotype.snp,
                             phenotype.type = "dichotomous",
                             model.type = "univariate",
                             mcmc.chains = 2,
                             mcmc.iterations = 1500,
                             mcmc.warmup = 500,
                             cores = 2,
                             hdi.level = 0.95,
                             stat.learn.method = "rf",
                             cv.iterations = 200,
                             rpa.iterations = 0,
                             with.stan.obj = FALSE)


###################################################
### code chunk number 29: genphenManual.Rnw:863-884
###################################################
# Get scores data
d.score <- d.out$scores

# Some optional formatting
d.score$label<-paste(d.score$site, ":", d.score$g1, "->", d.score$g0, sep = '')

# Visualization
g <- ggplot(data = d.score)+
  geom_errorbar(aes(x = ca, ymin = beta.L, ymax = beta.H),
                width = 0.015, col = "darkgray")+
  geom_point(aes(x = ca, y = beta.mean, fill = kappa), shape = 21, size = 4)+
  geom_text_repel(aes(x = ca, y = beta.mean, label = label), size = 5)+
  theme_bw(base_size = 14)+
  ylab(label = "Slope coefficient (with 95% HDI)")+
  scale_x_continuous(name = "CA", limits = c(0, 1.05))+
  geom_hline(yintercept = 0, linetype = "dashed")+
  theme(legend.position = "top")+
  scale_fill_distiller(palette = "Spectral", limits = c(-0.2, 1))+
  guides(fill = guide_colorbar(barwidth = 10, barheight = 1.5))

g


###################################################
### code chunk number 30: genphenManual.Rnw:891-892
###################################################
g


###################################################
### code chunk number 31: genphenManual.Rnw:903-922
###################################################
# Get the ppc output
d.ppc <- d.out$ppc

# Visualization
g <- ggplot(data = d.ppc)+
  geom_errorbar(aes(x = pred.mean.1, ymin = pred.1.L, ymax = pred.1.H),
                col = "darkgray")+
  geom_errorbar(aes(x = pred.mean.0, ymin = pred.0.L, ymax = pred.0.H),
                col = "darkgray")+
  geom_point(aes(x = pred.mean.1, y = real.mean.1/d.ppc$n1))+
  geom_point(aes(x = pred.mean.0, y = real.mean.0/d.ppc$n0))+
  geom_abline(slope = 1, intercept = 0, linetype = "dashed", col = "red")+
  xlim(c(0, 1))+
  ylim(c(0, 1))+
  ylab(label = "Predicted mean phenotype (with 95% HDI)")+
  xlab(label = "Real mean phenotype")+
  theme_bw(base_size = 14)

g


###################################################
### code chunk number 32: genphenManual.Rnw:929-930
###################################################
g


###################################################
### code chunk number 33: genphenManual.Rnw:971-977
###################################################
# Simulate 50,000 SNPs and 60 phenotypes
set.seed(seed = 551155)
g1 <- replicate(n=5*10^4, expr=as.character(rbinom(n=30, size = 1,prob = 0.49)))
g2 <- replicate(n=5*10^4, expr=as.character(rbinom(n=30, size = 1,prob = 0.51)))
gen <- rbind(g1, g2)
phen <- c(rnorm(n = 30, mean = 3, sd = 3), rnorm(n = 30, mean = 5, sd = 3))


###################################################
### code chunk number 34: genphenManual.Rnw:988-1012
###################################################
# Set diagnostic points (in range 1-50,000)
diagnostic.points <- c(seq(from = 0, to = 100, by = 20),
                       seq(from = 200, to = 1000, by = 200),
                       seq(from = 2000, to = 50000, by = 3000))
diagnostic.points[1] <- 1


# Run diagnostics
diag <- genphen::runDiagnostics(genotype = gen,
                                phenotype = phen,
                                phenotype.type = "continuous",
                                rf.trees = 50000,
                                mcmc.chains = 2,
                                mcmc.iterations = 1500,
                                mcmc.warmup = 500,
                                cores = 2,
                                hdi.level = 0.95,
                                diagnostic.points = diagnostic.points)

# Get diagnosis scores, and mark effects as significant|not-significant
diag.stats <- diag$scores
diag.stats$significant <- ifelse(test = diag.stats$beta.L <= 0
                                 & diag.stats$beta.H >= 0,
                                 yes = FALSE, no = TRUE)


###################################################
### code chunk number 35: genphenManual.Rnw:1022-1032
###################################################
# Visualization
g <- ggplot(data = diag$importance.scores)+
  geom_line(aes(x = importance.rank, y = importance))+
  xlab("Rank")+
  ylab("Importance")+
  theme_bw(base_size = 14)+
  scale_x_continuous(trans = "log10")+
  annotation_logticks(base = 10, sides = "b")

g


###################################################
### code chunk number 36: genphenManual.Rnw:1038-1039
###################################################
g


###################################################
### code chunk number 37: genphenManual.Rnw:1052-1069
###################################################
# Visualization
g <- ggplot(data = diag.stats)+
  geom_errorbar(aes(x = diagnostic.points, ymin = beta.L, ymax = beta.H),
                col = "gray", width = 0.05)+
  geom_point(aes(x = diagnostic.points, y = beta.mean, col = significant),
             size = 2)+
  geom_hline(yintercept = 0, linetype = "dashed", col = "gray")+
  theme_bw(base_size = 14)+
  xlab(label = "Rank")+
  ylab(label = "Slope coefficient (with 95% HDI)")+
  scale_x_continuous(trans = "log10", 
                     breaks = c(1, 10, 100, 10^3, 10^4,5*10^4),
                     labels = c(1, 10, 100, 10^3, 10^4, 5*10^4))+
  annotation_logticks(sides = "b")+
  theme(legend.position = "top")

g


###################################################
### code chunk number 38: genphenManual.Rnw:1075-1076
###################################################
g


